#include "testlib.h"
#include "bits/stdc++.h"

using namespace std;

#define all(x) x.begin(), x.end()

const int MAXC = 1000'000'000;
const int MAXN = 300'000;
const int MAXT = 10'000;

struct TestCase {
    int n;
    int m;
    vector<int> x;
    void Print() {
        cout << n << ' ' << m << '\n';
        for (int i = 0; i < m; i++) {
            cout << x[i];
            if (i != m - 1) {
                cout << ' ';
            } else {
                cout << '\n';
            }
        }
    }
};

vector<TestCase> tests;

const int ORDER_MODS_SIZE = 4;
const string order_mods[ORDER_MODS_SIZE] = {"random_mode", "sorted", "shuffled", "random_balance"};

const int NUMBERS_MODS_SIZE = 5;
const string numbers_mods[NUMBERS_MODS_SIZE] = {"random_mode", "equal", "different", "two_equal", "many_nulls"};

vector<int> GenNumbers(int n, int m, string numbers_mode, int min_c, int max_c) {
    vector<int> x;
    if (numbers_mode == "equal") {
        int c = rnd.next(min_c, max_c);
        x.assign(m, c);
    } else if (numbers_mode == "different") {
        x.assign(m, 0);
        for (int i = 0; i < m; ++i) {
            x[i] = rnd.next(min_c, max_c);
        }
    } else if (numbers_mode == "two_equal") {
        int c1 = rnd.next(min_c, max_c);
        int c2 = rnd.next(min_c, max_c);
        x.assign(m, 0);
        for (int i = 0; i < m; ++i) {
            if (rnd.next(1, 2) == 1) {
                x[i] = c1;
            } else {
                x[i] = c2;
            }
        }
    } else if (numbers_mode == "many_nulls") {
        x.assign(m, 0);
        for (int i = 0; i < min(n, m); i++) {
            x[i] = rnd.next(min_c, max_c);
        }
    }
    return x;
}

void GenOrder(int n, int m, vector<int> &x, string order_mode) {
    if (order_mode == "shuffled") {
        shuffle(all(x));
    } else if (order_mode == "sorted") {
        sort(all(x));
    } else if (order_mode == "random_balance") {
        vector<int> sorted_x = x;
        sort(all(x));
        vector<int> b(m);
        int cur_b = 0;
        int pluses = 0;
        for (int i = m - 1; i >= 0; i--) {
            if (cur_b == 0) {
                b[i] = 1;
                ++cur_b;
                ++pluses;
            } else if (cur_b == n) {
                b[i] = -1;
                --cur_b;
            } else {
                if (rnd.next(1, 2) == 1) {
                    b[i] = 1;
                    ++cur_b;
                    ++pluses;
                } else {
                    b[i] = -1;
                    --cur_b;
                }
            }
        }
        vector<int> low;
        vector<int> up;
        for (int i = 0; i < m - pluses; ++i) {
            low.push_back(x[i]);
        }
        for (int i = 0; i < pluses; ++i) {
            up.push_back(x[m - i - 1]);
        }
        shuffle(all(low));
        shuffle(all(up));
        int ptr1 = 0;
        int ptr2 = 0;
        for (int i = 0; i < m; ++i) {
            if (b[i] == 1) {
                x[i] = up[ptr1++];
            } else {
                x[i] = low[ptr2++];
            }
        }
    }
}

signed main(int argc, char* argv[]) {
    registerGen(argc, argv, 1);
    
    int max_t = opt<int>("max_t", MAXT);
    
    string order_mode = opt<string>("order_mode", "random_mode");
    string numbers_mode = opt<string>("numbers_mode", "random_mode");
    
    int min_n = opt<int>("min_n", 1);
    int max_n = opt<int>("max_n", MAXN);
    int max_sum_n = opt<int>("sum_n", MAXN);
    
    int min_m = opt<int>("min_m", 1);
    int max_m = opt<int>("max_m", MAXN);
    int max_sum_m = opt<int>("sum_m", MAXN);
    
    int min_c = opt<int>("min_c", - MAXC);
    int max_c = opt<int>("max_c", MAXC);
    
    int cur_sum_n = 0;
    int cur_sum_m = 0;
    
    for (int test_case = 1; test_case <= max_t; ++test_case) {
        int n = rnd.next(min_n, max_n);
        int m = rnd.next(min_m, max_m);
        if (cur_sum_n + n > max_sum_n) {
            break;
        }
        cur_sum_n += n;
        if (cur_sum_m + m > max_sum_m) {
            break;
        }
        cur_sum_m += m;

        string cur_numbers_mode = numbers_mode;
        if (cur_numbers_mode == "random_mode") {
            cur_numbers_mode = numbers_mods[rnd.next(1, NUMBERS_MODS_SIZE - 1)];
        }
        string cur_order_mode = order_mode;
        if (cur_order_mode == "random_mode") {
            cur_order_mode = order_mods[rnd.next(1, ORDER_MODS_SIZE - 1)];
        }
        
        auto x = GenNumbers(n, m, cur_numbers_mode, min_c, max_c);
        
        GenOrder(n, m, x, cur_order_mode);
        
        TestCase test{n, m, x};
        tests.push_back(test);
    }
    
    cout << tests.size() << '\n';
    for (auto &test : tests) {
        test.Print();
    }
}
